const {
  Scene,
  Object3D,
  PerspectiveCamera,
  WebGLRenderer,
  CanvasTexture,
  Sprite,
  SpriteMaterial,
  loadTexture,
  ImageUtils,
  PointLight,
  SpriteAlignment,
  shadowMap,
  BufferGeometry,
  BufferGeometryUtils,
  CubeTextureLoader,
  MultiMaterial,
  vertices
} = THREE;

import {TerrainGenerator} from './TerrainGenerator.js';
import { getData } from './getData.js';
import { Enum } from './Texture.js';
import fragmentShader from './fragmentShader';
import vertexShader from './vertexShader';

// Tweakables
const scaleFactor = 2; // HiDPI
const width = window.innerWidth * scaleFactor;
const height = window.innerHeight * scaleFactor;
const rotationSensitivity = 0.04;
const rotationIncrement = -0.2;


// Returns a color for a given datum and number of clusters.
const camera = new PerspectiveCamera(40, width / height, 1, 10000);
const renderer = new WebGLRenderer({ antialias: true, alpha: true
                                   });

renderer.setSize(width, height);
document.body.appendChild(renderer.domElement);
renderer.domElement.style.width = width / scaleFactor + 'px';
renderer.domElement.style.height = height / scaleFactor + 'px';



var controls = new THREE.OrbitControls(camera, renderer.domElement);


const scene = new Scene();

//GROUND
const groundWidth = 10;
const groundHeight = 10;
var groundTexture = new THREE.TextureLoader().load(
  'https://raw.githubusercontent.com/stemkoski/stemkoski.github.com/master/Three.js/images/checkerboard.jpg'
);
groundTexture.wrapS = groundTexture.wrapT = THREE.RepeatWrapping;
groundTexture.repeat.set(groundWidth/4, groundHeight/4);
var groundMaterial = new THREE.MeshBasicMaterial({
  map: groundTexture,
  side: THREE.DoubleSide
});
var groundGeometry = new THREE.PlaneGeometry(groundWidth, groundHeight);
var ground = new THREE.Mesh(groundGeometry, groundMaterial);
scene.add(ground);



const csvUrl = 'https://gist.githubusercontent.com/meperezcuello/6897ec1d8545ca3be077c694f738ab09/raw/e52a902376bd093b7b3765fb46b9a317add8de4f/WOW.csv';
const csvUrl2 = 'https://gist.githubusercontent.com/meperezcuello/eb1f6c7dabe63443aa1dafa169e51d07/raw/1c7e70db4548e8ce7eaf9c9ba21391681462651c/mountain.csv'
//** Custom Texture Mapping **//

let dict = [
    {texture:'oceanTexture', repeatedness: 5, smoothing: [0.01, 0.50]}, //<-- this is bottom texture
    {texture:'sandyTexture', repeatedness: 20, smoothing: [0.50, 0.55]},
    {texture:'snowyTexture', repeatedness: 20, smoothing: [0.55, 0.65]},
    {texture:'snowyTexture', repeatedness: 10, smoothing: [0.65, 0.75]},
    {texture:'snowyTexture', repeatedness: 20, smoothing: [0.75, 0.99]}, // <-- this is top texture
  ];

let wow = [
    {texture:'wowtest', repeatedness: 1, smoothing: [0.01, 0.01]}, //<-- this is bottom texture
  ];
  

const TerrainMain = async() => {
  
  const terrain =  new TerrainGenerator(csvUrl2,'testMapping')
  terrain.loadMaterials(wow);
  return await terrain.mesh()
  
}

(async() => {
  const mesh = await TerrainMain();
  console.log(mesh)
   mesh.rotation.x = -Math.PI / 2;
   mesh.position.y = 0;
 		scene.add(mesh);




  
  const waterCube = new THREE.BoxGeometry(61, 87, 4.5);
  
  const water = new THREE.Mesh(waterCube, new THREE.MeshBasicMaterial({color:0x3eaff3, transparent: true, opacity: 0.50, side: THREE.DoubleSide}));
 	scene.add(water);
water.rotation.x = -Math.PI / 2;
water.position.y = 1.00;
	

//Init ground roration
const epsilon = -0.5;
ground.rotation.x -= Math.PI / 2;
ground.position.z = groundWidth/2 + epsilon;
ground.position.x = groundHeight/2 + epsilon;
ground.position.y = epsilon;


var axesHelper = new THREE.AxesHelper( 100 );
scene.add( axesHelper );
//Init camera position
//camera.rotation.x = Math.PI/2;
camera.position.set(100, 100, -80);


const animate = () => {
  	//water.rotation.z -= 0.025;
   	//mesh.rotation.z -= 0.025;
    requestAnimationFrame(animate);
 		controls.update();
	  renderer.render(scene, camera);
};
  
  //dispose from memeory
//renderer.renderLists.dispose();
//mesh.geometry.dispose();
//mesh.material.dispose();
//scene.remove( mesh );

  
animate();
  })()